#pragma once

#include <string>
#include <memory>
#include <glm.hpp>
#include <HPModel.h>
#include <GLFW/glfw3.h>
#include "Renderer.h"

class Application
{
public:
    Application();
    ~Application();

    bool initialize(int width, int height, const std::string& title);
    void run();
    void loadModelFromJson(const std::string& jsonPath);
    void addSphere(int x, int y, int z, bool hydrophobic, bool hhContact);
    void initializeFromHPModel(const HPModel& model);
    void setImGuiStyle();

private:
    void initImGui();
    void cleanupImGui();
    void processInput();
    void update();
    void initializeRendererWithModel(const std::string& jsonStr);

    static void framebuffer_size_callback(GLFWwindow* window, int width, int height);
    static void mouse_callback(GLFWwindow* window, double xpos, double ypos);
    static void scroll_callback(GLFWwindow* window, double xoffset, double yoffset);

    GLFWwindow* m_window;
    int m_width;
    int m_height;
    std::string m_title;

    std::unique_ptr<Renderer> m_renderer;

    bool m_showUI;
    bool m_showGridPoints;
    bool m_firstMouse; // Add this variable to manage the first mouse interaction
    float m_deltaTime;
    float m_lastFrame;

    glm::vec3 m_cameraPos;
    glm::vec3 m_cameraFront;
    glm::vec3 m_cameraUp;
    float m_yaw;
    float m_pitch;
    float m_fov;

    char m_jsonInput[1024 * 16];
    std::string m_jsonErrorMessage;
    std::string m_errorMessage;

    bool m_darkMode;
};
