#include "InputHandler.h"
#include <glm.hpp>
#include <gtc/matrix_transform.hpp>

// Process keyboard input for camera movement
void InputHandler::processInput(GLFWwindow *window, float deltaTime,
                                glm::vec3& cameraPos, glm::vec3& cameraFront,
                                glm::vec3& cameraUp)
{
    float cameraSpeed = 10.0f * deltaTime;

    if (glfwGetKey(window, GLFW_KEY_W) == GLFW_PRESS)
        cameraPos += cameraSpeed * cameraFront;
    if (glfwGetKey(window, GLFW_KEY_S) == GLFW_PRESS)
        cameraPos -= cameraSpeed * cameraFront;
    if (glfwGetKey(window, GLFW_KEY_A) == GLFW_PRESS)
        cameraPos -= glm::normalize(glm::cross(cameraFront, cameraUp)) * cameraSpeed;
    if (glfwGetKey(window, GLFW_KEY_D) == GLFW_PRESS)
        cameraPos += glm::normalize(glm::cross(cameraFront, cameraUp)) * cameraSpeed;
    if (glfwGetKey(window, GLFW_KEY_SPACE) == GLFW_PRESS)
        cameraPos += cameraSpeed * cameraUp;
    if (glfwGetKey(window, GLFW_KEY_LEFT_SHIFT) == GLFW_PRESS)
        cameraPos -= cameraSpeed * cameraUp;

    if (glfwGetKey(window, GLFW_KEY_ESCAPE) == GLFW_PRESS)
        glfwSetWindowShouldClose(window, true);
}

// Handle key callbacks (e.g., toggling UI)
void InputHandler::key_callback(GLFWwindow* window, int key, int scancode, int action, int mods,
                                bool& showUI, bool& firstMouse)
{
    if (key == GLFW_KEY_TAB && action == GLFW_PRESS)
    {
        showUI = !showUI; // Toggle UI visibility
        if (showUI)
        {
            glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_NORMAL);
        }
        else
        {
            glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
            firstMouse = true; // Reset first mouse state
        }
    }
}
