#ifndef RENDERER_H
#define RENDERER_H

#include <glm.hpp>
#include <vector>
#include <map>
#include <tuple>
#include "Shader.h"
#include "Sphere.h"

class Renderer {
public:
    Renderer();
    ~Renderer();

    void initialize();
    void render(const glm::mat4& view, const glm::mat4& projection);
    void addSphere(const Sphere& sphere);
    void updateLines(const std::vector<std::tuple<int, int, int>>& sphereSequence);
    void setSphereScale(float scale);
    void setLineThickness(float thickness);
    void toggleGridPoints(bool show);

    // Accessors
    std::map<std::tuple<int, int, int>, Sphere>& getSpheres() { return spheres; }
    std::vector<std::tuple<int, int, int>>& getSphereSequence() { return sphereSequence; }

private:
    // Shader
    Shader* shader;

    // Sphere data
    unsigned int sphereVAO, sphereVBO, sphereEBO;
    std::vector<float> sphereVertices;
    std::vector<unsigned int> sphereIndices;

    // Lines data
    unsigned int lineVAO, lineVBO;
    std::vector<float> lineVertices;

    // Axes data
    unsigned int axesVAO, axesVBO;

    // Grid data
    unsigned int gridVAO, gridVBO;
    std::vector<float> gridPoints;
    bool showGridPointsFlag;

    // Line thickness and sphere scale
    float sphereScale;
    float lineThickness;

    // Spheres and sequence
    std::map<std::tuple<int, int, int>, Sphere> spheres;
    std::vector<std::tuple<int, int, int>> sphereSequence;

    // Helper functions
    void setupSphere();
    void setupLines();
    void setupAxes();
    void setupGrid();
    void updateLineVertices();
};

#endif
